@@ -1,8 +1,22 @@
+# Copyright 2022 Germán A. Prieto, MIT license
 """
-Module with all routines for multitaper 
-spectrum estimation (1D). 
-Contains:
-   mtspec - calculate the Thomson and/or Quadratic multitaper 
+Module with routines for univariate multitaper spectrum estimation (1D).
+Contains the main MTSpec and MTSine classes where the estimates 
+are made and stored.
+
+See module mtcross for bi-variate problems
+
+**Classes**
+
+   * MTSpec - A class to represent Thomson's multitaper estimates
+   * MTSine - A class to represent Sine Multitaper estimates
+
+**Functions**
+
+   * spectrogram - Computes a spectrogram with consecutive multitaper estimates.
+
+|
+
 """
 
 #-----------------------------------------------------
@@ -13,20 +27,156 @@ import numpy as np
 import scipy
 import scipy.signal as signal
 import scipy.linalg as linalg
-import matplotlib.pyplot as plt
 import multitaper.utils as utils 
-import time
 
 #-------------------------------------------------------------------------
 # MTSPEC main code 
 #-------------------------------------------------------------------------
 
-class mtspec:
+class MTSpec:
+
+    """
+
+    .. class:: MTSpec
+
+       A class for univariate Thomson multitaper estimates
+
+    **Attibutes**
+
+       *Parameters*
+          npts   : int
+             number of points of time series
+          nfft   : int
+             number of points of FFT. Dafault adds padding. 
+          nw     : flaot
+             time-bandwidth product
+          kspec  : int
+             number of tapers to use
+
+       *Time series*
+          x      : ndarray [npts]
+             time series
+          xvar   : float
+             variance of time series
+          dt     : float
+             sampling interval
+
+       *Frequency vector*
+          nf     : int
+             number of unique frequency points of spectral 
+             estimate, assuming real time series
+          freq   : ndarray [nfft]
+             frequency vector in Hz
+          df     : float
+             frequncy sampling interval
+
+       *Method*
+          iadapt : int
+             defines methos to use
+             0 - adaptive multitaper
+             1 - unweighted, wt =1 for all tapers
+             2 - wt by the eigenvalue of DPSS
+          ireal  : int
+             0 - real time series
+             1 - complex time series
+
+       *DPSS tapers and eigenvalues*
+          vn     : ndarray [npts,kspec]
+             Slepian sequences
+          lamb   : ndarray [kspec]
+             Eigenvalues of Slepian sequences
+
+       *Spectral estimates*
+          yk     : complex ndarray [nfft,kspec]
+             eigencoefficients, fft of tapered series
+          sk     : ndarray [nfft,kspec]
+             eigenspectra, power spectra for each yk
+          spec   : ndarray [nfft,1] 
+             multitaper estimate
+          se     : ndarray [nfft,1] 
+             degrees of freedom of estimate
+          wt     : ndarray [nfft,kspec]
+             weights for each eigencoefficient at each frequency
+
+    **Methods**
+
+       - init     : Constructor of the MTSpec class
+       - rspec    : returns the positive frequency of the spectra only
+       - reshape  : reshape yk's based on F-test of line components
+       - jackspec : estimate 95% confidence interval of multitaper estimate
+       - qiinv    : the quadratic inverse spectral estimate
+       - ftest    : F-test of line components in the spectra
+       - df_spec  : dual-frequency autospectra 
+
+
+    **References**
+    
+    Based on David J. Thomson's codes, Alan Chave and Thomson's Codes and
+    partial codes from EISPACK, Robert L. Parker and Glenn Ierley from
+    Scripps Institution of Oceanography. And my own Fortran90 library.
+
+    **Notes**
+
+    The class is in charge of estimating the adaptive weigthed 
+    multitaper spectrum, as in Thomson 1982. 
+    This is done by estimating the dpss (discrete prolate spheroidal 
+    sequences), multiplying each of the kspec tapers with the data 
+    series, take the fft, and using the adaptive scheme for a better
+    estimation. 
+   
+    As a by product of the spectrum (spec), all intermediate steps 
+    are retained, which can be used for bi-variate analysis, deconvolotuion, 
+    returning to the time domain, etc. 
+    By-products include the complex information in yk, the eigenspectra sk, 
+    the jackknife 95% confidence intervals (spec_ci), the degrees 
+    of freedom (se) and the weigths wt(nf,kspec) used.
+    See below for a complete list. 
+    
+
+    **Modified**
+
+	January 2022 (German Prieto)
+
+   |
+
+   """
 
     def __init__(self,x,nw=4,kspec=0,dt=1.0,nfft=0,iadapt=0,vn=None,lamb=None):
         """
-        put some notes here
-        Only works so far, on a single dimensional direction
+        The constructor of the **MTSpec** class.
+        
+        It performs main steps in multitaper estimation, saving the
+        MTSpec class variable with attributes described above.
+        
+        To use for first time given a time series `x`:
+    
+            psd = MTSpec(x,nw,kspec,dt,iadapt)
+        
+        *Parameters*
+        
+           x : ndarray [npts,]
+              Time series to analyze
+           nw : float, optional
+              time bandwidth product, default = 4
+           kspec : int, optional
+              number of tapers, default = 2*nw-1
+           dt : float, optional
+              sampling interval of x, default = 1.0
+           nfft : int, optional
+              number of frequency points for FFT, allowing for padding
+              default = 2*npts+1
+           iadapt : int, optional
+              defines methos to use, default = 0
+              0 - adaptive multitaper
+              1 - unweighted, wt =1 for all tapers
+              2 - wt by the eigenvalue of DPSS
+           vn : ndarray [npts,kspec], optional
+              Slepian sequences, can be precomputed to save time
+           lamb : ndarray [kspec], optional
+              Eigenvalues of DPSS, can be precomputed to save time
+
+        |
+
         """
 
         #-----------------------------------------------------
@@ -157,13 +307,17 @@ class mtspec:
 
     def rspec(self,*args):
         """
-        If input signal is real, returns the positive frequencies 
-        only. It keeps the power correct (twice power in positive 
-        frequencies, except at freq=0 and freq=fnyq) 
+        Returns the spetra at positive frequencies, checking that 
+        a real input signal was used.
+
+        *Parameters*
         
-        If input is another vector (qispec, spec_ci, etc) then 
-        only that vector is processed. 
-        Vector must be real (must be power spectrum or derivatives)
+           args : ndarray
+              another array to return the positive frequencies. 
+              Could be qispec, spec_ci, etc.
+
+        |
+
         """
 
         nargs = len(args)
@@ -213,9 +367,40 @@ class mtspec:
     # Remove lines, save spectrum without lines
     #----------------------------------------------------------------
 
-    def reshape(self,fcrit=0.95,p=None):   #p_in,yk_in,vn,fcrit=0.95):
+    def reshape(self,fcrit=0.95,p=None):   
         """
-        reshape the yk's based on the F-test of line compenents
+        Reshape eigenft's (yk) around significant spectral lines
+        The "significant" means above fcritical probability (0.95)
+        If probability is large at neighbouring frequencies, I will 
+        only remove the largest probability energy. 
+
+        Returns recalculated yk, sk, spec, wt, and se
+
+        *Parameters*
+        
+           fcrit : float optional
+              Probability value over which to reshape, default = 0.95
+           p : ndarray optional [nfft] 
+              F-test probabilities to find fcritical
+              If None, it will be calculated
+
+        *Returns*
+        
+           respec : ndarray [nfft]
+              The reshaped PSD estimate 
+           spec : ndarray [nfft]
+              the PSD without the line components
+           yk : ndarray [nfft,kspec]
+              the eigenft's without line components
+           sline : ndarray [nfft]
+              the PSD of the line components only
+
+        *Calls*
+        
+           utils.yk_reshape
+
+        |
+
         """
 
         if (p is None):
@@ -254,6 +439,21 @@ class mtspec:
 
     def jackspec(self):
 
+        """
+        code to calculate adaptively weighted jackknifed 95% confidence limits
+
+        *Returns*
+        
+           spec_ci : ndarray [nfft,2]
+              real array of jackknife error estimates, with 5 and 95%
+              confidence intervals of the spectrum.
+
+        *Calls*
+        
+           utils.jackspec
+
+        """
+
         spec_ci = utils.jackspec(self.spec,self.sk,self.wt,self.se)
 
         return spec_ci 
@@ -266,13 +466,41 @@ class mtspec:
     # qiinv
     #-------------------------------------------------------------------------
     
-    def qiinv(self): #spec,yk,wt,vn,lamb,nw):
-    
+    def qiinv(self): 
         """
         Function to calculate the Quadratic Spectrum using the method 
         developed by Prieto et al. (2007).   
+        
         The first 2 derivatives of the spectrum are estimated and the 
         bias associated with curvature (2nd derivative) is reduced. 
+
+        Calculate the Stationary Inverse Theory Spectrum.
+        Basically, compute the spectrum inside the innerband. 
+  
+        This approach is very similar to D.J. Thomson (1990).
+
+        *Returns*
+        
+           qispec : ndarray [nfft,0]
+              the QI spectrum estimate
+           ds : ndarray [nfft,0]	
+              the estimate of the first derivative
+           dds : ndarray [nfft,0]	
+              the estimate of the second derivative
+
+        *References*
+        
+           G. A. Prieto, R. L. Parker, D. J. Thomson, F. L. Vernon, 
+           and R. L. Graham (2007), Reducing the bias of multitaper 
+           spectrum estimates,  Geophys. J. Int., 171, 1269-1281. 
+           doi: 10.1111/j.1365-246X.2007.03592.x.
+  
+        *Calls*
+        
+           utils.qiinv
+        
+        | 
+
         """
    
         qispec, ds, dds = utils.qiinv(self.spec,self.yk,
@@ -300,9 +528,26 @@ class mtspec:
     # ftest
     #-------------------------------------------------------------------------
     
-    def ftest(self): #vn,yk):
+    def ftest(self):
         """
         Performs the F test for a line component
+
+        Computes F-test for single spectral line components
+        at the frequency bins given in the MTSpec class.
+
+        **Returns**
+        
+  	    F : ndarray [nfft]
+            vector of f test values, real
+  	    p : ndarray [nfft]
+            vector with probability of line component
+
+        **Calls**
+        
+        utils.f_test
+        
+        |
+        
         """
    
         F,p   = utils.ftest(self.vn, self.yk)
@@ -320,6 +565,22 @@ class mtspec:
     def df_spec(self):
         """
         Performs the dual-frequency spectrum of a signal with itself.
+
+        *Returns*
+        
+           df_spec : ndarray complex, 2D (nf,nf)
+              the complex dual-frequency cross-spectrum. Not normalized
+           df_cohe : ndarray, 2D (nf,nf)
+              MSC, dual-freq coherence matrix. Normalized (0.0,1.0)
+           df_phase : ndarray, 2D (nf,nf)
+              the dual-frequency phase
+
+        *Calls*
+        
+           utils.df_spec
+
+        |
+
         """
    
         df_spec, df_cohe, df_phase = utils.df_spec(self)
@@ -338,26 +599,97 @@ class mtspec:
 # CLASS MTSINE
 #------------------------------------------------------------------------------
 
-class mtsine:
+class MTSine:
+
+    """
+    .. class:: MTSpec
+
+       A class for univariate Thomson multitaper estimates
+
+    **Attibutes**
+
+    *Parameters*
+       npts   : int
+          number of points of time series
+       nfft   : int
+          number of points of FFT. nfft = 2*npts
+
+    *Time series*
+       x      : ndarray [npts]
+          time series
+       xvar   : float
+          variance of time series
+       dt     : float
+          sampling interval
+
+    *Frequency vector*
+       nf     : int
+          number of unique frequency points of spectral 
+          estimate, assuming real time series
+       freq   : ndarray [nfft]
+          frequency vector in Hz
+       df     : float
+          frequncy sampling interval
+
+    *Method*
+       ntap   : int
+          fixed number of tapers
+          if ntap<0, use kopt
+       kopt   : ndarray [nfft,1] 
+          number of tapers at each frequency
+       ntimes : int
+          number of max iterations to perform
+       ireal  : int
+          0 - real time series
+          1 - complex time series
+
+    *Spectral estimates*
+       spec   : ndarray [nfft,1] 
+          multitaper estimate
+       err     : ndarray [nfft,2]
+          1-std confidence interval of spectral estimate
+          simple dof estimate
+
+    **Notes**
+    
+    The class is in charge of estimating the adaptive sine 
+    multitaper as in Riedel and Sidorenko (1995). 
+    This is done by performing a MSE adaptive estimation. First
+    a pilot spectral estimate is used, and S" is estimated, in 
+    order to get te number of tapers to use, using (13) of 
+    R & S for a min square error spectrum.
+
+    |
+
+    """
 
     def __init__(self,x,ntap=0,ntimes=0,fact=1.0,dt=1.0):
         """
-        Performs the PSD estimation by the sie multitaper method of
-        Riedel and Sidorenko, IEEE Tr. Sig. Pr, 43, 188, 1995
+        Performs the PSD estimation by the sine multitaper method
+     
+        **Parameters**
+        
+           x : ndarray [npts]	
+              real, data vector
+      	   ntap : int, optional
+              constant number of tapers (def = 0)
+      	   ntimes : int, optional
+              number of iterations to perform
+      	   fact : float, optional
+              degree of smoothing (def = 1.)
+       	   dt : float
+              sampling interval of time series
       
-        Based on Bob Parker psd.f codes. Most of the comments come 
-        his documentation as well.
-      
-        Last Modified:
-      	German Prieto
-      	September 22 2005
-      	
-        The subroutine is in charge of estimating the adaptive sine 
+        
+        **Notes**
+        
+        This function is in charge of estimating the adaptive sine 
         multitaper as in Riedel and Sidorenko (1995). 
         This is done by performing a MSE adaptive estimation. First
         a pilot spectral estimate is used, and S" is estimated, in 
         order to get te number of tapers to use, using (13) of 
         R & S for a min square error spectrum. 
+
         Unlike the prolate spheroidal multitapers, the sine multitaper 
         adaptive process introduces a variable resolution and error in 
         the frequency domain. Complete error information is contained 
@@ -386,28 +718,29 @@ class mtsine:
         restricts the rate of growth of the number of tapers so that a 
         neighboring covering interval estimate is never completely 
         contained in the next such interval.
+
         This method SHOULD not be used for sharp cutoffs or deep 
         valleys, or small sample sizes. Instead use Thomson multitaper
         in mtspec in this same library. 
-      
-        INPUT
-      	  x[npts]	real, data vector
-      	  ntap		integer, constant number of tapers (def = 0)
-      	  ntimes	integer, number of iterations to perform
-      	  fact		real, degree of smoothing (def = 1.)
-       	  dt		real, sampling rate of time series
-      
-        OUTPUT
-      	  freq(nfft)	real vector with frequency bins
-      	  spec(nfft)	real vector with the adaptive estimated spectrum
-        Note: NFFT is 2*NPTS
 
-        OPTIONAL OUTPUT
-          kopt(nf)	integer, number of taper per freq point
-      	  err(nf,2)	1-std errors (simple dof estimate)
+        **References**
+        
+        Riedel and Sidorenko, IEEE Tr. Sig. Pr, 43, 188, 1995
+
+        Based on Bob Parker psd.f codes. Most of the comments come 
+        his documentation as well.
+
       	
-        calls quick, adapt
-      
+        **Modified**
+    
+      	September 22 2005
+
+
+        **Calls**
+        
+        utils.quick, utils.adapt
+     
+        | 
         """
 
         #-----------------------------------------------------
@@ -550,4 +883,133 @@ class mtsine:
 # end CLASS MTSINE
 #------------------------------------------------------------------------------
 
+#------------------------------------------------------------------------------
+# Functions
+#------------------------------------------------------------------------------
 
+#--------------------------
+# Spectrogram
+#--------------------------
+
+def spectrogram(data,dt,twin,olap=0.5,nw=3.5,kspec=5,fmin=0.0,fmax=-1.0,iadapt=0):
+    """
+    Computes a spectrogram with consecutive multitaper estimates.
+    Returns both Thomson's multitaper and the Quadratic multitaper estimate
+
+    **Parameters**
+    
+    data : array_like (npts,)
+        Time series or sequence
+    dt : float
+        Sampling interval in seconds of the time series.
+    twin : float
+        Time duration in seconds of each segment for a single multitaper estimate.
+    olap : float, optional
+        Overlap requested for the segment in building the spectrogram. 
+        Defaults = 0.5, values must be (0.0 - 0.99).
+        Overlap rounds to the nearest integer point. 
+    nw : float, optional
+        Time-bandwidth product for Thomson's multitaper algorithm.
+        Default = 3.5
+    kspec : int, optional
+        Number of tapers for avearaging the multitaper estimate.
+        Default = 5
+    fmin : float, optional
+        Minimum frequency to estimate the spectrogram, otherwise returns the 
+        entire spectrogram matrix.
+        Default = 0.0 Hz
+    fmax : float, optional
+        Maximum frequency to estimate the spectrogram, otherwise returns the
+        entire spectrogram matrix. 
+        Default = 0.5/dt Hz (Nyquist frequency)
+    iadapt : integer, optional
+        User defined, determines which method for multitaper averaging to use. 
+        Default = 0
+        0 - Adaptive multitaper
+        1 - Eigenvalue weights
+        2 - Constant weighting
+
+    **Returns**
+    
+    f : ndarray
+        Array of sample frequencies.
+    t : ndarray
+        Array of segment times.
+    Quad : ndarray
+        Spectrogram of x using the quadratic multitaper estimate.
+    MT : ndarray
+        Spectrogram of x using Thomson's multitaper estimate.
+    
+    By default, the last axis of Quad/MT corresponds to the segment times.
+
+    **See Also**
+    
+    MTSpec: Multitaper estimate of a time series.
+
+    **Notes**
+    
+    The code assumes a real input signals and thus mainly returns the positive 
+    frequencies. For a complex input signals, code qould require adaptation.  
+
+    **References**
+    
+       Prieto, G.A. (2022). The multitaper spectrum analysis package in Python.
+       Seism. Res. Lett In review. 
+
+    **Examples**
+   
+    To do
+
+    |
+
+    """
+    
+    if (fmax<=0.0):
+        fmax = 0.5/dt
+    
+    nwin  = int(np.round(twin/dt))
+    if (olap<=0.0):
+        njump = nwin
+    else:
+        njump = int(np.round(twin*(1.0-olap))/dt)
+
+    npts  = np.size(data)
+    nmax  = npts-nwin
+    nvec  = np.arange(0,nmax,njump)
+    t     = nvec*dt
+    nspec = len(nvec)
+
+    print('Window length %5.1fs and overlap %2.0f%%' %(twin, olap*100))
+    print('Total number of spectral estimates', nspec)
+    print('Frequency band of interest (%5.2f-%5.2f)Hz' %(fmin, fmax))
+
+    vn,theta = utils.dpss(nwin,nw,kspec)
+    for i in range(nspec):
+        if ((i+1)%10==0):
+            print('Loop ',i+1,' of ',nspec)
+
+        i1  = nvec[i]
+        i2  = i1+nwin
+        x   = data[i1:i2+1]
+
+        psd = MTSpec(x,nw,kspec,dt,iadapt=iadapt,
+                            vn=vn,lamb=theta)
+
+        freq2   = psd.freq
+        spec    = psd.spec 
+        qispec  = psd.qiinv()[0]   
+
+        nf         = len(freq2)
+
+        if (i==0):
+            fres   = np.where((freq2>=fmin) & (freq2<=fmax))[0]
+            nf     = len(fres)
+            f      = freq2[fres]
+            Quad   = np.zeros((nf,nspec),dtype=float)
+            MT     = np.zeros((nf,nspec),dtype=float)
+            print('Total frequency points %i' %(nf))
+
+        Quad[:,i]  = qispec[fres,0]
+        MT[:,i]    = spec[fres,0] 
+    
+    return t,f,Quad,MT
